@@ -1,8 +1,7 @@
 /* gdbmconst.h - The constants defined for use in gdbm. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1990-1991, 1993, 2007, 2011, 2013, 2016-2020 Free
-   Software Foundation, Inc.
+   Copyright (C) 1990-2021 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,7 +51,7 @@
 #define BUCKET_AVAIL 6
 
 /* The size of the bucket cache. */
-#define DEFAULT_CACHESIZE  100
+#define DEFAULT_CACHESIZE  GDBM_CACHE_AUTO
 
 #ifndef SIZE_T_MAX
 /* Maximum size representable by a size_t variable */
