@@ -34,11 +34,12 @@
 /* Maximum value for off_t */
 #define OFF_T_MAX SIGNED_TYPE_MAXIMUM (off_t)
 
-/* Return true if A can be added to B without integer overflow */
-static inline off_t
+/* Return true if both A and B are non-negative offsets and A can be added
+   to B without integer overflow */
+static inline int
 off_t_sum_ok (off_t a, off_t b)
 {
-  return OFF_T_MAX - a >= b;
+  return a >= 0 && b >= 0 && OFF_T_MAX - a >= b;
 }
 
 /* The type definitions are next.  */
@@ -271,6 +272,7 @@ struct gdbm_file_info
   off_t  mapped_pos;     /* Current offset in the region */
   off_t  mapped_off;     /* Position in the file where the region
 			    begins */
+  int mmap_preread :1;   /* 1 if prefault reading is requested */
 };
 
 #define GDBM_DIR_COUNT(db) ((db)->header->dir_size / sizeof (off_t))
