@@ -1,8 +1,7 @@
 /* gdbmerrno.c - convert gdbm errors into english. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1993, 2007, 2011, 2013, 2016-2020 Free Software
-   Foundation, Inc.
+   Copyright (C) 1993-2021 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -134,12 +133,15 @@ const char * const gdbm_errlist[_GDBM_MAX_ERRNO+1] = {
   [GDBM_DIR_OVERFLOW]           = N_("Bucket directory overflow"),
   [GDBM_BAD_BUCKET]             = N_("Malformed bucket header"),
   [GDBM_BAD_HEADER]             = N_("Malformed database file header"),
+  /* TRANSLATORS: avail_block is a field name. Don't translate it. */
   [GDBM_BAD_AVAIL]              = N_("Malformed avail_block"),
   [GDBM_BAD_HASH_TABLE]         = N_("Malformed hash table"),
   [GDBM_BAD_DIR_ENTRY]          = N_("Invalid directory entry"),
   [GDBM_FILE_CLOSE_ERROR]       = N_("Error closing file"),
   [GDBM_FILE_SYNC_ERROR]        = N_("Error synchronizing file"),
-  [GDBM_FILE_TRUNCATE_ERROR]    = N_("Error truncating file")
+  [GDBM_FILE_TRUNCATE_ERROR]    = N_("Error truncating file"),
+  [GDBM_BUCKET_CACHE_CORRUPTED] = N_("Bucket cache corrupted"),
+  [GDBM_BAD_HASH_ENTRY]         = N_("Malformed bucket hash entry")
 };
 
 const char *
