@@ -39,6 +39,8 @@ compute_directory_size (blksize_t block_size,
   int dir_size = 8 * sizeof (off_t);
   int dir_bits = 3;
 
+  if (block_size > INT_MAX / 2)
+    block_size = INT_MAX / 2;
   while (dir_size < block_size && dir_bits < GDBM_HASH_BITS - 3)
     {
       dir_size <<= 1;
@@ -554,6 +556,7 @@ gdbm_fd_open (int fd, const char *file_name, int block_size,
 #if HAVE_MMAP
   if (!(flags & GDBM_NOMMAP))
     {
+      dbf->mmap_preread = (flags & GDBM_PREREAD) != 0;
       if (_gdbm_mapped_init (dbf) == 0)
 	dbf->memory_mapping = TRUE;
       else
