@@ -65,8 +65,7 @@
 #line 1 "gram.y" /* yacc.c:339  */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1990-1991, 1993, 2007, 2011, 2013, 2016-2020 Free
-   Software Foundation, Inc.
+   Copyright (C) 1990-2021 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -87,7 +86,7 @@
 struct dsegm *dsdef[DS_MAX];
   
 
-#line 91 "gram.c" /* yacc.c:339  */
+#line 90 "gram.c" /* yacc.c:339  */
 
 # ifndef YY_NULLPTR
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -153,7 +152,7 @@ extern int yydebug;
 
 union YYSTYPE
 {
-#line 50 "gram.y" /* yacc.c:355  */
+#line 49 "gram.y" /* yacc.c:355  */
 
   char *string;
   struct kvpair *kvpair;
@@ -167,7 +166,7 @@ union YYSTYPE
   struct { struct dsegm *head, *tail; } dsegmlist;
   struct command *cmd;
 
-#line 171 "gram.c" /* yacc.c:355  */
+#line 170 "gram.c" /* yacc.c:355  */
 };
 
 typedef union YYSTYPE YYSTYPE;
@@ -198,7 +197,7 @@ int yyparse (void);
 
 /* Copy the second part of user declarations.  */
 
-#line 202 "gram.c" /* yacc.c:358  */
+#line 201 "gram.c" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -498,12 +497,12 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,    66,    66,    67,    70,    71,    74,    78,    83,    84,
-      85,    95,    95,   107,   108,   112,   115,   118,   122,   129,
-     133,   139,   145,   149,   164,   165,   172,   176,   182,   186,
-     194,   195,   198,   198,   206,   210,   216,   217,   220,   235,
-     239,   247,   251,   255,   260,   267,   271,   272,   275,   276,
-     279,   311,   339,   340,   343
+       0,    65,    65,    66,    69,    70,    73,    77,    82,    83,
+      84,    94,    94,   106,   107,   111,   114,   117,   121,   128,
+     132,   138,   144,   148,   163,   164,   171,   175,   181,   185,
+     193,   194,   197,   197,   205,   209,   215,   216,   219,   241,
+     245,   253,   257,   261,   266,   273,   277,   278,   281,   282,
+     285,   317,   345,   346,   349
 };
 #endif
 
@@ -1419,24 +1418,24 @@ yyreduce:
   switch (yyn)
     {
         case 6:
-#line 75 "gram.y" /* yacc.c:1646  */
+#line 74 "gram.y" /* yacc.c:1646  */
     {
 	      run_last_command ();
 	    }
-#line 1427 "gram.c" /* yacc.c:1646  */
+#line 1426 "gram.c" /* yacc.c:1646  */
     break;
 
   case 7:
-#line 79 "gram.y" /* yacc.c:1646  */
+#line 78 "gram.y" /* yacc.c:1646  */
     {
 	      if (run_command ((yyvsp[-2].cmd), &(yyvsp[-1].arglist)) && !interactive ())
 		exit (EXIT_USAGE);
 	    }
-#line 1436 "gram.c" /* yacc.c:1646  */
+#line 1435 "gram.c" /* yacc.c:1646  */
     break;
 
   case 10:
-#line 86 "gram.y" /* yacc.c:1646  */
+#line 85 "gram.y" /* yacc.c:1646  */
     {
 	      if (interactive ())
 		{
@@ -1446,17 +1445,17 @@ yyreduce:
 	      else
 		YYERROR;
 	    }
-#line 1450 "gram.c" /* yacc.c:1646  */
+#line 1449 "gram.c" /* yacc.c:1646  */
     break;
 
   case 11:
-#line 95 "gram.y" /* yacc.c:1646  */
+#line 94 "gram.y" /* yacc.c:1646  */
     { end_def(); }
-#line 1456 "gram.c" /* yacc.c:1646  */
+#line 1455 "gram.c" /* yacc.c:1646  */
     break;
 
   case 12:
-#line 96 "gram.y" /* yacc.c:1646  */
+#line 95 "gram.y" /* yacc.c:1646  */
     {
 	      if (interactive ())
 		{
@@ -1466,68 +1465,68 @@ yyreduce:
 	      else
 		YYERROR;
 	    }
-#line 1470 "gram.c" /* yacc.c:1646  */
+#line 1469 "gram.c" /* yacc.c:1646  */
     break;
 
   case 15:
-#line 112 "gram.y" /* yacc.c:1646  */
+#line 111 "gram.y" /* yacc.c:1646  */
     {
 	      gdbmarglist_init (&(yyval.arglist), NULL);
 	    }
-#line 1478 "gram.c" /* yacc.c:1646  */
+#line 1477 "gram.c" /* yacc.c:1646  */
     break;
 
   case 17:
-#line 119 "gram.y" /* yacc.c:1646  */
+#line 118 "gram.y" /* yacc.c:1646  */
     {
 	      gdbmarglist_init (&(yyval.arglist), (yyvsp[0].arg));
 	    }
-#line 1486 "gram.c" /* yacc.c:1646  */
+#line 1485 "gram.c" /* yacc.c:1646  */
     break;
 
   case 18:
-#line 123 "gram.y" /* yacc.c:1646  */
+#line 122 "gram.y" /* yacc.c:1646  */
     {
 	      gdbmarglist_add (&(yyvsp[-1].arglist), (yyvsp[0].arg));
 	      (yyval.arglist) = (yyvsp[-1].arglist);
 	    }
-#line 1495 "gram.c" /* yacc.c:1646  */
+#line 1494 "gram.c" /* yacc.c:1646  */
     break;
 
   case 19:
-#line 130 "gram.y" /* yacc.c:1646  */
+#line 129 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.arg) = gdbmarg_string ((yyvsp[0].string), &(yylsp[0]));
 	    }
-#line 1503 "gram.c" /* yacc.c:1646  */
+#line 1502 "gram.c" /* yacc.c:1646  */
     break;
 
   case 20:
-#line 134 "gram.y" /* yacc.c:1646  */
+#line 133 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.arg) = gdbmarg_kvpair ((yyvsp[0].kvpair), &(yylsp[0]));
 	    }
-#line 1511 "gram.c" /* yacc.c:1646  */
+#line 1510 "gram.c" /* yacc.c:1646  */
     break;
 
   case 21:
-#line 140 "gram.y" /* yacc.c:1646  */
+#line 139 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.kvpair) = (yyvsp[-1].kvlist).head;
 	    }
-#line 1519 "gram.c" /* yacc.c:1646  */
+#line 1518 "gram.c" /* yacc.c:1646  */
     break;
 
   case 22:
-#line 146 "gram.y" /* yacc.c:1646  */
+#line 145 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.kvlist).head = (yyval.kvlist).tail = (yyvsp[0].kvpair);
 	    }
-#line 1527 "gram.c" /* yacc.c:1646  */
+#line 1526 "gram.c" /* yacc.c:1646  */
     break;
 
   case 23:
-#line 150 "gram.y" /* yacc.c:1646  */
+#line 149 "gram.y" /* yacc.c:1646  */
     {
 	      if (kvlist_find ((yyvsp[-2].kvlist).head, (yyvsp[0].kvpair)->key))
 		{
@@ -1540,163 +1539,170 @@ yyreduce:
 	      (yyvsp[-2].kvlist).tail = (yyvsp[0].kvpair);
 	      (yyval.kvlist) = (yyvsp[-2].kvlist);
 	    }
-#line 1544 "gram.c" /* yacc.c:1646  */
+#line 1543 "gram.c" /* yacc.c:1646  */
     break;
 
   case 25:
-#line 166 "gram.y" /* yacc.c:1646  */
+#line 165 "gram.y" /* yacc.c:1646  */
     {
 	      (yyvsp[0].kvpair)->key = (yyvsp[-2].string);
 	      (yyval.kvpair) = (yyvsp[0].kvpair);
 	    }
-#line 1553 "gram.c" /* yacc.c:1646  */
+#line 1552 "gram.c" /* yacc.c:1646  */
     break;
 
   case 26:
-#line 173 "gram.y" /* yacc.c:1646  */
+#line 172 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.kvpair) = kvpair_string (&(yylsp[0]), (yyvsp[0].string));
 	    }
-#line 1561 "gram.c" /* yacc.c:1646  */
+#line 1560 "gram.c" /* yacc.c:1646  */
     break;
 
   case 27:
-#line 177 "gram.y" /* yacc.c:1646  */
+#line 176 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.kvpair) = kvpair_list (&(yylsp[-2]), (yyvsp[-1].slist).head);
 	    }
-#line 1569 "gram.c" /* yacc.c:1646  */
+#line 1568 "gram.c" /* yacc.c:1646  */
     break;
 
   case 28:
-#line 183 "gram.y" /* yacc.c:1646  */
+#line 182 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.slist).head = (yyval.slist).tail = slist_new_s ((yyvsp[0].string));
 	    }
-#line 1577 "gram.c" /* yacc.c:1646  */
+#line 1576 "gram.c" /* yacc.c:1646  */
     break;
 
   case 29:
-#line 187 "gram.y" /* yacc.c:1646  */
+#line 186 "gram.y" /* yacc.c:1646  */
     {
 	      struct slist *s = slist_new_s ((yyvsp[0].string));
 	      slist_insert (&(yyvsp[-2].slist).tail, s);
 	      (yyval.slist) = (yyvsp[-2].slist);
 	    }
-#line 1587 "gram.c" /* yacc.c:1646  */
+#line 1586 "gram.c" /* yacc.c:1646  */
     break;
 
   case 32:
-#line 198 "gram.y" /* yacc.c:1646  */
+#line 197 "gram.y" /* yacc.c:1646  */
     { begin_def (); }
-#line 1593 "gram.c" /* yacc.c:1646  */
+#line 1592 "gram.c" /* yacc.c:1646  */
     break;
 
   case 33:
-#line 199 "gram.y" /* yacc.c:1646  */
+#line 198 "gram.y" /* yacc.c:1646  */
     {
 	      end_def ();
 	      dsegm_list_free (dsdef[(yyvsp[-2].num)]);
 	      dsdef[(yyvsp[-2].num)] = (yyvsp[0].dsegm);
 	    }
-#line 1603 "gram.c" /* yacc.c:1646  */
+#line 1602 "gram.c" /* yacc.c:1646  */
     break;
 
   case 34:
-#line 207 "gram.y" /* yacc.c:1646  */
+#line 206 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.dsegm) = (yyvsp[-2].dsegmlist).head;
 	    }
-#line 1611 "gram.c" /* yacc.c:1646  */
+#line 1610 "gram.c" /* yacc.c:1646  */
     break;
 
   case 35:
-#line 211 "gram.y" /* yacc.c:1646  */
+#line 210 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.dsegm) = dsegm_new_field ((yyvsp[0].type), NULL, 1);
 	    }
-#line 1619 "gram.c" /* yacc.c:1646  */
+#line 1618 "gram.c" /* yacc.c:1646  */
     break;
 
   case 38:
-#line 221 "gram.y" /* yacc.c:1646  */
+#line 220 "gram.y" /* yacc.c:1646  */
     {
 	      if (strcmp ((yyvsp[0].string), "key") == 0)
-		(yyval.num) = DS_KEY;
+		{
+		  (yyval.num) = DS_KEY;
+		  free ((yyvsp[0].string));
+		}
 	      else if (strcmp ((yyvsp[0].string), "content") == 0)
-		(yyval.num) = DS_CONTENT;
+		{
+		  (yyval.num) = DS_CONTENT;
+		  free ((yyvsp[0].string));
+		}
 	      else
 		{
 		  terror (_("expected \"key\" or \"content\", "
 			    "but found \"%s\""), (yyvsp[0].string));
+		  free ((yyvsp[0].string));
 		  YYERROR;
 		}
 	    }
-#line 1636 "gram.c" /* yacc.c:1646  */
+#line 1642 "gram.c" /* yacc.c:1646  */
     break;
 
   case 39:
-#line 236 "gram.y" /* yacc.c:1646  */
+#line 242 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.dsegmlist).head = (yyval.dsegmlist).tail = (yyvsp[0].dsegm);
 	    }
-#line 1644 "gram.c" /* yacc.c:1646  */
+#line 1650 "gram.c" /* yacc.c:1646  */
     break;
 
   case 40:
-#line 240 "gram.y" /* yacc.c:1646  */
+#line 246 "gram.y" /* yacc.c:1646  */
     {
 	      (yyvsp[-2].dsegmlist).tail->next = (yyvsp[0].dsegm);
 	      (yyvsp[-2].dsegmlist).tail = (yyvsp[0].dsegm);
 	      (yyval.dsegmlist) = (yyvsp[-2].dsegmlist);
 	    }
-#line 1654 "gram.c" /* yacc.c:1646  */
+#line 1660 "gram.c" /* yacc.c:1646  */
     break;
 
   case 41:
-#line 248 "gram.y" /* yacc.c:1646  */
+#line 254 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.dsegm) = dsegm_new_field ((yyvsp[-1].type), (yyvsp[0].string), 1);
 	    }
-#line 1662 "gram.c" /* yacc.c:1646  */
+#line 1668 "gram.c" /* yacc.c:1646  */
     break;
 
   case 42:
-#line 252 "gram.y" /* yacc.c:1646  */
+#line 258 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.dsegm) = dsegm_new_field ((yyvsp[-4].type), (yyvsp[-3].string), (yyvsp[-1].num));
 	    }
-#line 1670 "gram.c" /* yacc.c:1646  */
+#line 1676 "gram.c" /* yacc.c:1646  */
     break;
 
   case 43:
-#line 256 "gram.y" /* yacc.c:1646  */
+#line 262 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.dsegm) = dsegm_new (FDEF_OFF);
 	      (yyval.dsegm)->v.n = (yyvsp[0].num);
 	    }
-#line 1679 "gram.c" /* yacc.c:1646  */
+#line 1685 "gram.c" /* yacc.c:1646  */
     break;
 
   case 44:
-#line 261 "gram.y" /* yacc.c:1646  */
+#line 267 "gram.y" /* yacc.c:1646  */
     {
 	      (yyval.dsegm) = dsegm_new (FDEF_PAD);
 	      (yyval.dsegm)->v.n = (yyvsp[0].num);
 	    }
-#line 1688 "gram.c" /* yacc.c:1646  */
+#line 1694 "gram.c" /* yacc.c:1646  */
     break;
 
   case 45:
-#line 268 "gram.y" /* yacc.c:1646  */
+#line 274 "gram.y" /* yacc.c:1646  */
     {
 	      variable_print_all (stdout);
             }
-#line 1696 "gram.c" /* yacc.c:1646  */
+#line 1702 "gram.c" /* yacc.c:1646  */
     break;
 
   case 50:
-#line 280 "gram.y" /* yacc.c:1646  */
+#line 286 "gram.y" /* yacc.c:1646  */
     {
 	      int t = 1;
 	      int rc;
@@ -1728,11 +1734,11 @@ yyreduce:
 		}
 	      free ((yyvsp[0].string));
 	    }
-#line 1732 "gram.c" /* yacc.c:1646  */
+#line 1738 "gram.c" /* yacc.c:1646  */
     break;
 
   case 51:
-#line 312 "gram.y" /* yacc.c:1646  */
+#line 318 "gram.y" /* yacc.c:1646  */
     {
 	      int rc = variable_set ((yyvsp[-2].string), VART_STRING, (yyvsp[0].string));
 	      switch (rc)
@@ -1758,11 +1764,11 @@ yyreduce:
 	      free ((yyvsp[-2].string));
 	      free ((yyvsp[0].string));
 	    }
-#line 1762 "gram.c" /* yacc.c:1646  */
+#line 1768 "gram.c" /* yacc.c:1646  */
     break;
 
   case 54:
-#line 344 "gram.y" /* yacc.c:1646  */
+#line 350 "gram.y" /* yacc.c:1646  */
     {
 	      int rc = variable_unset ((yyvsp[0].string));
 	      switch (rc)
@@ -1780,11 +1786,11 @@ yyreduce:
 		}
 	      free ((yyvsp[0].string));
 	    }
-#line 1784 "gram.c" /* yacc.c:1646  */
+#line 1790 "gram.c" /* yacc.c:1646  */
     break;
 
 
-#line 1788 "gram.c" /* yacc.c:1646  */
+#line 1794 "gram.c" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2019,7 +2025,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 363 "gram.y" /* yacc.c:1906  */
+#line 369 "gram.y" /* yacc.c:1906  */
 
 
 void
