@@ -1,8 +1,7 @@
 /* proto.h - The prototypes for the dbm routines. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1990-1991, 1993, 2007, 2011, 2013, 2017-2020 Free
-   Software Foundation, Inc.
+   Copyright (C) 1990-2021 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,16 +20,19 @@
 /* From bucket.c */
 void _gdbm_new_bucket	(GDBM_FILE, hash_bucket *, int);
 int _gdbm_get_bucket	(GDBM_FILE, int);
-int _gdbm_read_bucket_at (GDBM_FILE dbf, off_t off, hash_bucket *bucket,
-			  size_t size);
+int _gdbm_fetch_data   (GDBM_FILE dbf, off_t off, size_t size, void *buf);
 
 int _gdbm_split_bucket (GDBM_FILE, int);
 int _gdbm_write_bucket (GDBM_FILE, cache_elem *);
+int _gdbm_cache_init   (GDBM_FILE, size_t);
+void _gdbm_cache_free  (GDBM_FILE dbf);
+int _gdbm_cache_flush  (GDBM_FILE dbf);
 
 /* From falloc.c */
 off_t _gdbm_alloc       (GDBM_FILE, int);
 int  _gdbm_free         (GDBM_FILE, off_t, int);
 void _gdbm_put_av_elem  (avail_elem, avail_elem [], int *, int);
+int _gdbm_avail_block_read (GDBM_FILE dbf, avail_block *avblk, size_t size);
 
 /* From findkey.c */
 char *_gdbm_read_entry  (GDBM_FILE, int);
@@ -47,14 +49,10 @@ int _gdbm_end_update   (GDBM_FILE);
 void _gdbm_fatal	(GDBM_FILE, const char *);
 
 /* From gdbmopen.c */
-int _gdbm_init_cache	(GDBM_FILE, size_t);
-void _gdbm_cache_entry_invalidate (GDBM_FILE, int);
-
-int gdbm_avail_block_validate (GDBM_FILE dbf, avail_block *avblk);
-int gdbm_bucket_avail_table_validate (GDBM_FILE dbf, hash_bucket *bucket);
-
 int _gdbm_validate_header (GDBM_FILE dbf);
 
+int _gdbm_file_size (GDBM_FILE dbf, off_t *psize);
+
 /* From mmap.c */
 int _gdbm_mapped_init	(GDBM_FILE);
 void _gdbm_mapped_unmap	(GDBM_FILE);
@@ -86,6 +84,29 @@ int _gdbm_dump (GDBM_FILE dbf, FILE *fp);
 /* From recover.c */
 int _gdbm_next_bucket_dir (GDBM_FILE dbf, int bucket_dir);
 
+/* cachetree.c */
+cache_tree *_gdbm_cache_tree_alloc (void);
+void _gdbm_cache_tree_destroy (cache_tree *tree);
+void _gdbm_cache_tree_delete (cache_tree *tree, struct cache_node *n);
+
+/* avail.c */
+int gdbm_avail_block_validate (GDBM_FILE dbf, avail_block *avblk, size_t size);
+int gdbm_bucket_avail_table_validate (GDBM_FILE dbf, hash_bucket *bucket);
+int gdbm_avail_traverse (GDBM_FILE dbf,
+			 int (*cb) (avail_block *, off_t, void *),
+			 void *data);
+
+
+/* Return codes for _gdbm_cache_tree_lookup. */
+enum
+  {
+    node_found,   /* Returned element was found in cache. */
+    node_new,     /* Returned element has been created and inserted to cache */
+    node_failure  /* An error occurred. */
+  };
+
+int _gdbm_cache_tree_lookup (cache_tree *tree, off_t adr, cache_node **retval);
+
 /* I/O functions */
 static inline ssize_t
 gdbm_file_read (GDBM_FILE dbf, void *buf, size_t size)
@@ -128,6 +149,7 @@ gdbm_file_sync (GDBM_FILE dbf)
       GDBM_SET_ERRNO (dbf, GDBM_FILE_SYNC_ERROR, TRUE);
       return 1;
     }
+  return 0;
 #else
   sync ();
   sync ();
