@@ -1,6 +1,5 @@
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1990-1991, 1993, 2007, 2011, 2013, 2017-2020 Free
-   Software Foundation, Inc.
+   Copyright (C) 1990-2021 Free Software Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -96,6 +95,7 @@ vgetyn (const char *prompt, va_list ap)
 		case 'N':
 		  return 0;
 		default:
+		  /* TRANSLATORS: Please, don't translate 'y' and 'n'. */
 		  fprintf (stdout, "%s\n", _("Please, reply 'y' or 'n'"));
 		}
 	      /* fall through */
